@@ -3,13 +3,11 @@ These are classes to support contour plotting and labelling for the Axes class.
 """
 
 from numbers import Integral
-import warnings
 
 import numpy as np
 from numpy import ma
 
 import matplotlib as mpl
-import matplotlib._contour as _contour
 import matplotlib.path as mpath
 import matplotlib.ticker as ticker
 import matplotlib.cm as cm
@@ -197,7 +195,7 @@ class ContourLabeler(object):
 
         self.labelXYs = []
 
-        if cbook.iterable(self.labelManual):
+        if np.iterable(self.labelManual):
             for x, y in self.labelManual:
                 self.add_label_near(x, y, inline,
                                     inline_spacing)
@@ -216,11 +214,11 @@ class ContourLabeler(object):
         self.labelTextsList = cbook.silent_list('text.Text', self.labelTexts)
         return self.labelTextsList
 
-    cl = property(cbook.deprecated("3.0", alternative="labelTexts")(
+    cl = cbook.deprecated("3.0", alternative="labelTexts")(property(
         lambda self: self.labelTexts))
-    cl_xy = property(cbook.deprecated("3.0", alternative="labelXYs")(
+    cl_xy = cbook.deprecated("3.0", alternative="labelXYs")(property(
         lambda self: self.labelXYs))
-    cl_cvalues = property(cbook.deprecated("3.0", alternative="labelCValues")(
+    cl_cvalues = cbook.deprecated("3.0", alternative="labelCValues")(property(
         lambda self: self.labelCValues))
 
     def print_label(self, linecontour, labelwidth):
@@ -230,11 +228,9 @@ class ContourLabeler(object):
 
     def too_close(self, x, y, lw):
         "Return *True* if a label is already near this location."
-        for loc in self.labelXYs:
-            d = np.sqrt((x - loc[0]) ** 2 + (y - loc[1]) ** 2)
-            if d < 1.2 * lw:
-                return True
-        return False
+        thresh = (1.2 * lw) ** 2
+        return any((x - loc[0]) ** 2 + (y - loc[1]) ** 2 < thresh
+                   for loc in self.labelXYs)
 
     def get_label_coords(self, distances, XX, YY, ysize, lw):
         """
@@ -267,7 +263,7 @@ class ContourLabeler(object):
         if not isinstance(lev, str):
             lev = self.get_text(lev, fmt)
 
-        lev, ismath = text.Text.is_math_text(lev)
+        lev, ismath = text.Text()._preprocess_math(lev)
         if ismath == 'TeX':
             if not hasattr(self, '_TeX_manager'):
                 self._TeX_manager = texmanager.TexManager()
@@ -285,33 +281,6 @@ class ContourLabeler(object):
 
         return lw
 
-    @cbook.deprecated("2.2")
-    def get_real_label_width(self, lev, fmt, fsize):
-        """
-        This computes actual onscreen label width.
-        This uses some black magic to determine onscreen extent of non-drawn
-        label.  This magic may not be very robust.
-
-        This method is not being used, and may be modified or removed.
-        """
-        # Find middle of axes
-        xx = np.mean(np.asarray(self.ax.axis()).reshape(2, 2), axis=1)
-
-        # Temporarily create text object
-        t = text.Text(xx[0], xx[1])
-        self.set_label_props(t, self.get_text(lev, fmt), 'k')
-
-        # Some black magic to get onscreen extent
-        # NOTE: This will only work for already drawn figures, as the canvas
-        # does not have a renderer otherwise.  This is the reason this function
-        # can't be integrated into the rest of the code.
-        bbox = t.get_window_extent(renderer=self.ax.figure.canvas.renderer)
-
-        # difference in pixel extent of image
-        lw = np.diff(bbox.corners()[0::2, 0])[0]
-
-        return lw
-
     def set_label_props(self, label, text, color):
         """Set the label properties - color, fontsize, text."""
         label.set_text(text)
@@ -468,7 +437,7 @@ class ContourLabeler(object):
     def _get_label_clabeltext(self, x, y, rotation):
         # x, y, rotation is given in pixel coordinate. Convert them to
         # the data coordinate and create a label using ClabelText
-        # class. This way, the roation of the clabel is along the
+        # class. This way, the rotation of the clabel is along the
         # contour line always.
         transDataInv = self.ax.transData.inverted()
         dx, dy = transDataInv.transform_point((x, y))
@@ -506,7 +475,7 @@ class ContourLabeler(object):
         """
         # x, y, rotation is given in pixel coordinate. Convert them to
         # the data coordinate and create a label using ClabelText
-        # class. This way, the roation of the clabel is along the
+        # class. This way, the rotation of the clabel is along the
         # contour line always.
 
         t = self._get_label_clabeltext(x, y, rotation)
@@ -704,8 +673,10 @@ def _is_closed_polygon(X):
 
 def _find_closest_point_on_path(lc, point):
     """
-    lc: coordinates of vertices
-    point: coordinates of test point
+    Parameters
+    ----------
+    lc : coordinates of vertices
+    point : coordinates of test point
     """
 
     # find index of closest vertex for this segment
@@ -772,22 +743,22 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         particular contour level are grouped together so that
         ``level0segs = [polygon0]`` and ``level0kinds = [polygon0kinds]``.
 
-    kwargs :
+    **kwargs
         Keyword arguments are as described in the docstring of
         `~.axes.Axes.contour`.
 
     Attributes
     ----------
-    ax:
+    ax
         The axes object in which the contours are drawn.
 
-    collections:
+    collections
         A silent_list of LineCollections or PolyCollections.
 
-    levels:
+    levels
         Contour levels.
 
-    layers:
+    layers
         Same as levels for line contours; half-way between
         levels for filled contours.  See :meth:`_process_colors`.
     """
@@ -808,7 +779,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         Parameters
         ----------
-        ax :
+        ax : `~.axes.Axes`
             The `~.axes.Axes` object to draw on.
 
         levels : [level0, level1, ..., leveln]
@@ -857,9 +828,8 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         if self.antialiased is None and self.filled:
             self.antialiased = False  # eliminate artifacts; we are not
                                       # stroking the boundaries.
-            # The default for line contours will be taken from
-            # the LineCollection default, which uses the
-            # rcParams['lines.antialiased']
+            # The default for line contours will be taken from the
+            # LineCollection default, which uses :rc:`lines.antialiased`.
 
         self.nchunk = kwargs.pop('nchunk', 0)
         self.locator = kwargs.pop('locator', None)
@@ -871,9 +841,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         else:
             self.logscale = False
 
-        if self.origin not in [None, 'lower', 'upper', 'image']:
-            raise ValueError("If given, *origin* must be one of [ 'lower' |"
-                             " 'upper' | 'image']")
+        cbook._check_in_list([None, 'lower', 'upper', 'image'], origin=origin)
         if self.extent is not None and len(self.extent) != 4:
             raise ValueError("If given, *extent* must be '[ *None* |"
                              " (x0,x1,y0,y1) ]'")
@@ -939,7 +907,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         if self.filled:
             if self.linewidths is not None:
-                warnings.warn('linewidths is ignored by contourf')
+                cbook._warn_external('linewidths is ignored by contourf')
 
             # Lower and upper contour levels.
             lowers, uppers = self._get_lowers_and_uppers()
@@ -996,8 +964,8 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         if kwargs:
             s = ", ".join(map(repr, kwargs))
-            warnings.warn('The following kwargs were not used by contour: ' +
-                          s)
+            cbook._warn_external('The following kwargs were not used by '
+                                 'contour: ' + s)
 
     def get_transform(self):
         """
@@ -1098,7 +1066,6 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         self.allkinds = len(args) > 2 and args[2] or None
         self.zmax = np.max(self.levels)
         self.zmin = np.min(self.levels)
-        self._auto = False
 
         # Check lengths of levels and allsegs.
         if self.filled:
@@ -1184,7 +1151,6 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         one contour line, but two filled regions, and therefore
         three levels to provide boundaries for both regions.
         """
-        self._auto = True
         if self.locator is None:
             if self.logscale:
                 self.locator = ticker.LogLocator()
@@ -1218,11 +1184,6 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         """
         Determine the contour levels and store in self.levels.
         """
-        if self.filled:
-            fn = 'contourf'
-        else:
-            fn = 'contour'
-        self._auto = False
         if self.levels is None:
             if len(args) == 0:
                 levels_arg = 7  # Default, hard-wired.
@@ -1240,8 +1201,8 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
             levels_in = self.levels[inside]
             if len(levels_in) == 0:
                 self.levels = [self.zmin]
-                warnings.warn("No contour levels were found"
-                              " within the data range.")
+                cbook._warn_external(
+                    "No contour levels were found within the data range.")
 
         if self.filled and len(self.levels) < 2:
             raise ValueError("Filled contours require at least 2 levels.")
@@ -1337,7 +1298,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         if linewidths is None:
             tlinewidths = [(mpl.rcParams['lines.linewidth'],)] * Nlev
         else:
-            if not cbook.iterable(linewidths):
+            if not np.iterable(linewidths):
                 linewidths = [linewidths] * Nlev
             else:
                 linewidths = list(linewidths)
@@ -1363,7 +1324,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         else:
             if isinstance(linestyles, str):
                 tlinestyles = [linestyles] * Nlev
-            elif cbook.iterable(linestyles):
+            elif np.iterable(linestyles):
                 tlinestyles = list(linestyles)
                 if len(tlinestyles) < Nlev:
                     nreps = int(np.ceil(Nlev / len(linestyles)))
@@ -1458,16 +1419,16 @@ class QuadContourSet(ContourSet):
 
     Attributes
     ----------
-    ax:
+    ax
         The axes object in which the contours are drawn.
 
-    collections:
+    collections
         A silent_list of LineCollections or PolyCollections.
 
-    levels:
+    levels
         Contour levels.
 
-    layers:
+    layers
         Same as levels for line contours; half-way between
         levels for filled contours. See :meth:`_process_colors` method.
     """
@@ -1486,6 +1447,8 @@ class QuadContourSet(ContourSet):
             self._mins = args[0]._mins
             self._maxs = args[0]._maxs
         else:
+            import matplotlib._contour as _contour
+
             self._corner_mask = kwargs.pop('corner_mask', None)
             if self._corner_mask is None:
                 self._corner_mask = mpl.rcParams['contour.corner_mask']
@@ -1558,7 +1521,8 @@ class QuadContourSet(ContourSet):
         self.zmin = float(z.min())
         if self.logscale and self.zmin <= 0:
             z = ma.masked_where(z <= 0, z)
-            warnings.warn('Log scale: values of z <= 0 have been masked')
+            cbook._warn_external('Log scale: values of z <= 0 have been '
+                                 'masked')
             self.zmin = float(z.min())
         self._contour_level_args(z, args)
         return (x, y, z)
@@ -1665,11 +1629,9 @@ class QuadContourSet(ContourSet):
 
             contour([X, Y,] Z, [levels], **kwargs)
 
-        :func:`~matplotlib.pyplot.contour` and
-        :func:`~matplotlib.pyplot.contourf` draw contour lines and
-        filled contours, respectively.  Except as noted, function
-        signatures and return values are the same for both versions.
-
+        `.contour` and `.contourf` draw contour lines and filled contours,
+        respectively.  Except as noted, function signatures and return values
+        are the same for both versions.
 
         Parameters
         ----------
@@ -1677,7 +1639,7 @@ class QuadContourSet(ContourSet):
             The coordinates of the values in *Z*.
 
             *X* and *Y* must both be 2-D with the same shape as *Z* (e.g.
-            created via :func:`numpy.meshgrid`), or they must both be 1-D such
+            created via `numpy.meshgrid`), or they must both be 1-D such
             that ``len(X) == M`` is the number of columns in *Z* and
             ``len(Y) == N`` is the number of rows in *Z*.
 
@@ -1709,8 +1671,7 @@ class QuadContourSet(ContourSet):
             nearest those points are always masked out, other triangular
             corners comprising three unmasked points are contoured as usual.
 
-            Defaults to ``rcParams['contour.corner_mask']``, which defaults to
-            ``True``.
+            Defaults to :rc:`contour.corner_mask`, which defaults to ``True``.
 
         colors : color string or sequence of colors, optional
             The colors of the levels, i.e. the lines for `.contour` and the
@@ -1756,19 +1717,17 @@ class QuadContourSet(ContourSet):
             - 'lower': ``Z[0, 0]`` is at X=0.5, Y=0.5 in the lower left corner.
             - 'upper': ``Z[0, 0]`` is at X=N+0.5, Y=0.5 in the upper left
               corner.
-            - 'image': Use the value from :rc:`image.origin`. Note: The value
-              *None* in the rcParam is currently handled as 'lower'.
+            - 'image': Use the value from :rc:`image.origin`.
 
         extent : (x0, x1, y0, y1), optional
-            If *origin* is not *None*, then *extent* is interpreted as
-            in :func:`matplotlib.pyplot.imshow`: it gives the outer
-            pixel boundaries. In this case, the position of Z[0,0]
-            is the center of the pixel, not a corner. If *origin* is
-            *None*, then (*x0*, *y0*) is the position of Z[0,0], and
-            (*x1*, *y1*) is the position of Z[-1,-1].
+            If *origin* is not *None*, then *extent* is interpreted as in
+            `.imshow`: it gives the outer pixel boundaries. In this case, the
+            position of Z[0,0] is the center of the pixel, not a corner. If
+            *origin* is *None*, then (*x0*, *y0*) is the position of Z[0,0],
+            and (*x1*, *y1*) is the position of Z[-1,-1].
 
-            This keyword is not active if *X* and *Y* are specified in
-            the call to contour.
+            This argument is ignored if *X* and *Y* are specified in the call
+            to contour.
 
         locator : ticker.Locator subclass, optional
             The locator is used to determine the contour levels if they
@@ -1797,7 +1756,7 @@ class QuadContourSet(ContourSet):
                 properties of its colormap are changed. Therefore, an explicit
                 call `.QuadContourSet.changed()` is needed after modifying the
                 colormap. The explicit call can be left out, if a colorbar is
-                assigned to the `.QuadContourSet` because it interally calls
+                assigned to the `.QuadContourSet` because it internally calls
                 `.QuadContourSet.changed()`.
 
             Example::
@@ -1816,7 +1775,7 @@ class QuadContourSet(ContourSet):
             Override axis units by specifying an instance of a
             :class:`matplotlib.units.ConversionInterface`.
 
-        antialiased : bool, optinal
+        antialiased : bool, optional
             Enable antialiasing, overriding the defaults.  For
             filled contours, the default is *True*.  For line contours,
             it is taken from :rc:`lines.antialiased`.
@@ -1862,20 +1821,17 @@ class QuadContourSet(ContourSet):
             Hatching is supported in the PostScript, PDF, SVG and Agg
             backends only.
 
-
         Notes
         -----
-        1. :func:`~matplotlib.pyplot.contourf` differs from the MATLAB
-           version in that it does not draw the polygon edges.
-           To draw edges, add line contours with
-           calls to :func:`~matplotlib.pyplot.contour`.
+        1. `.contourf` differs from the MATLAB version in that it does not draw
+           the polygon edges. To draw edges, add line contours with calls to
+           `.contour`.
 
-        2. contourf fills intervals that are closed at the top; that
-           is, for boundaries *z1* and *z2*, the filled region is::
+        2. `.contourf` fills intervals that are closed at the top; that is, for
+           boundaries *z1* and *z2*, the filled region is::
 
               z1 < Z <= z2
 
-           There is one exception: if the lowest boundary coincides with
-           the minimum value of the *Z* array, then that minimum value
-           will be included in the lowest interval.
+           except for the lowest interval, which is closed on both sides (i.e.
+           it includes the lowest value).
         """
